/* Copyright (c) 2000-2008 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

if (typeof(ko) == 'undefined') {
    ko = {};
}
if (typeof(ko.formatters) == 'undefined') {
    ko.formatters = {};
}

/* Create the ko.formatters scope. */
(function() {

    /************************************************************
     *        Globals                                           *
     ************************************************************/

    var log = ko.logging.getLogger('ko.formatters');
    //log.setLevel(ko.logging.LOG_DEBUG);
    const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    const FORMAT_SELECTION = Components.interfaces.koIFormatterContext.FORMAT_SELECTION;
    const FORMAT_BLOCK = Components.interfaces.koIFormatterContext.FORMAT_BLOCK;
    const FORMAT_TEXT = Components.interfaces.koIFormatterContext.FORMAT_TEXT;
    // Variable to hold the number of highlighting events that have occurred.
    var highlight_count = 0;

    /************************************************************
     *         Controller for the formatter commands            *
     ************************************************************/

    // Hide this controller from our global namespace, we simply dont need to
    // touch anything in it from the outside world.
    function FormattersController() {
    }

    // The following two lines ensure proper inheritance (see Flanagan, p. 144).
    FormattersController.prototype = new Controller();
    FormattersController.prototype.constructor = FormattersController;

    FormattersController.prototype.destructor = function() {
    }

    FormattersController.prototype.is_cmd_format_enabled = function () {
        var view = ko.views.manager.currentView;
        return view && view.getAttribute('type') == 'editor';
    };

    FormattersController.prototype.do_cmd_format = function () {
        ko.formatters.formatView(ko.views.manager.currentView);
    };

    FormattersController.prototype.is_cmd_formatSubLanguageBlock_enabled = function () {
        var view = ko.views.manager.currentView;
        return view && view.getAttribute('type') == 'editor';
    };

    FormattersController.prototype.do_cmd_formatSubLanguageBlock = function () {
        ko.formatters.formatSubLanguageBlock(ko.views.manager.currentView);
    };

    /************************************************************
     *        Internal private functions                        *
     ************************************************************/

    /**
     * Format the supplied view with the provided formatter and context.
     
     * @param view {Components.interfaces.koIScintillaView}  The view to format.
     * @param formatter {Components.interfaces.koIFormatter}  Formatter to use.
     * @param context {Components.interfaces.koIFormatterContext}  Context to use.
     */
    function _formatViewWithFormatterAndContext(view, formatter, context) {
        try {
            // perLineWhitespace - how much whitespace needs to be added to
            //                     each formatted line.
            var perLineWhitespace = "";
            // firstLineWhitespace - how much whitespace was already in the
            //                          first line.
            var firstLineWhitespace = "";
            var preceedingSelectionWhitespace = "";
            if (context.type == FORMAT_SELECTION ||
                context.type == FORMAT_BLOCK) {
                // Remember the preceeding indentation.
                var match = context.text.match(/^(\s+)/);
                if (match) {
                    perLineWhitespace = match[1];
                    // Be careful when dealing with multiple lines, use the
                    // last line of the regex match, as that is first line that
                    // contains real text.
                    var lines = perLineWhitespace.split(/\r\n|\n|\r/g);
                    if (lines.length > 1) {
                        perLineWhitespace = lines[lines.length - 1];
                    }
                }
            }

            // Remember the original settings, so we can compare to later.
            var pretext = context.text;
            var scimoz = view.scimoz;
            var original_currentPos = scimoz.currentPos;
            var original_anchor = scimoz.anchor;
            var original_line = scimoz.lineFromPosition(original_currentPos);
            var original_lineCount = scimoz.lineCount;
            var original_firstVisibleLine = scimoz.firstVisibleLine;

            try {
                formatter.format(context);
            } catch (ex) {
                ko.statusBar.AddMessage(ex, "formatting", 5000, true);
                return;
            }
            var text = context.text;
            // Formatting worked, update the text.
            if (context.type == FORMAT_SELECTION ||
                context.type == FORMAT_BLOCK) {
                if (context.type == FORMAT_SELECTION) {
                    var selPos = Math.min(original_currentPos, original_anchor);
                    var startLinePos = scimoz.positionFromLine(scimoz.lineFromPosition(selPos));
                    if (startLinePos < selPos) {
                        // Selection does not begin at the start of the line.
                        var preceedingText = scimoz.getTextRange(startLinePos, selPos);
                        var match = preceedingText.match(/^(\s+)/);
                        if (match && (match[1] == preceedingText)) {
                            // The preceeding text is just whitespace, need to
                            // maintain the indentation here as well.
                            preceedingSelectionWhitespace = preceedingText;
                            firstLineWhitespace = perLineWhitespace;
                            perLineWhitespace = preceedingText + perLineWhitespace;
                        }
                    }
                } else {
                    // Select the block of text to be replaced.
                    scimoz.selectionMode = scimoz.SC_SEL_STREAM;
                    scimoz.anchor = context.pos_start;
                    scimoz.currentPos = context.pos_end;
                }
                if (perLineWhitespace) {
                    // Update all lines to include this preceeding whitespace.
                    var lines = text.split(/\r\n|\n|\r/g);
                    for (var i=0; i < lines.length; i++) {
                        /* Only indent lines that have some text. */
                        if (lines[i]) {
                            if (i == 0 && preceedingSelectionWhitespace) {
                                // The first line already has some preceeding
                                // whitespace from outside the user's selection.
                                // Only need to add in part of the whitespace.
                                lines[i] = firstLineWhitespace + lines[i];
                            } else {
                                lines[i] = perLineWhitespace + lines[i];
                            }
                        }
                    }

                    // Re-piece together the text.
                    text = lines.join("\n");

                    // Ensure the last line EOL state does not change.
                    var pretextEndsWithNewLine = pretext[pretext.length - 1].match(/\r|\n/);
                    var textEndsWithNewLine = text[text.length - 1].match(/\r|\n/);
                    if (!pretextEndsWithNewLine && textEndsWithNewLine) {
                        // The original text did not end with a newline, so
                        // the resulting text should not either.
                        text = text.substr(0, text.length - 1);
                    } else if (pretextEndsWithNewLine && !textEndsWithNewLine) {
                        // The original text ended with a newline, so the
                        // resulting text should as well.
                        text += "\n";
                    }
                }
            } else {
                // All the text is going to get replaced by insertText().
                scimoz.selectAll();
            }
            view.insertText(text);

            // Restore the line and possiblity the position as best as possible.
            var new_lineCount = scimoz.lineCount;
            var line_difference = new_lineCount - original_lineCount;
            if (context.type == FORMAT_SELECTION) {
                // This will maintain the selection over the formatted area.
                if (original_currentPos <= original_anchor) {
                    scimoz.currentPos = original_currentPos;
                } else {
                    scimoz.anchor = original_anchor;
                }
            } else if (context.type == FORMAT_BLOCK) {
                // Just put it back in the original location, ensuring the
                // cursor does not leave the block if it was within the block
                // before the format operation.
                var offSet = ko.stringutils.bytelength(text) - ko.stringutils.bytelength(pretext);
                var new_currentPos = original_currentPos + offSet;
                var new_anchor = original_anchor + offSet;
                if (original_currentPos >= context.pos_start &&
                    original_currentPos <= context.pos_end) {
                    new_currentPos = Math.max(new_currentPos, context.pos_start);
                    new_currentPos = Math.min(new_currentPos, context.pos_end);
                }
                if (original_anchor >= context.pos_start &&
                    original_anchor <= context.pos_end) {
                    new_anchor = Math.max(new_anchor, context.pos_start);
                    new_anchor = Math.min(new_anchor, context.pos_end);
                }
                scimoz.currentPos = new_currentPos;
                scimoz.anchor = new_anchor;
            } else {
                // Just maintain the same line number, which may or may not be
                // the same line after the replacement.
                var new_line = original_line + line_difference;
                if (new_line < 0) {
                    new_line = 0;
                }
                var new_currentPos = scimoz.positionFromLine(new_line);
                scimoz.currentPos = new_currentPos;
                scimoz.anchor = new_currentPos;
            }

            // Ensure the caret is still visible.
            scimoz.scrollCaret();
            var new_firstVisibleLine = scimoz.firstVisibleLine;
            if (original_firstVisibleLine != new_firstVisibleLine) {
                scimoz.lineScroll(0, original_firstVisibleLine - new_firstVisibleLine);
            }

        } catch (ex) {
            log.exception(ex);
        }
    };

    /**
     * Highlight this formatting region in the current view.
     * @param {int} pos_start - The scimoz starting position.
     * @param {int} pos_end - The scimoz end position.
     */
    this.highlightRegion = function(pos_start, pos_end) {
        //dump("highlightRegion\n");
        highlight_count += 1;
        var view = ko.views.manager.currentView;
        if (!view) {
            return;
        }
        var scimoz = view.scimoz;
        scimoz.indicatorCurrent = Components.interfaces.koILintResult.DECORATOR_TAG_MATCH_HIGHLIGHT;
        scimoz.indicatorFillRange(pos_start, pos_end - pos_start);
    }

    /**
     * Remove the formatter highlighting from the current view.
     */
    this.unhighlightRegion = function() {
        //dump("unhighlightRegion\n");
        highlight_count -= 1;
        //dump('highlight_count: ' + highlight_count + '\n');
        if (highlight_count > 0) {
            // This unhighlighting event occured after a call to re-highlight
            // the region, so we do not want to remove the highlighting now.
            return;
        }
        var view = ko.views.manager.currentView;
        if (!view) {
            return;
        }
        var scimoz = view.scimoz;
        scimoz.indicatorCurrent = Components.interfaces.koILintResult.DECORATOR_TAG_MATCH_HIGHLIGHT;
        scimoz.indicatorClearRange(0, scimoz.length);
    }

        /* Add in formatter menus/menuitems for the formatter configurations. */
    function _createMenuitems(menupopup, strbundle, formatter_configurations, region_name, sublang, pos_start, pos_end, addHighlighting)
    {
        var formatterSvc = Components.classes["@activestate.com/koFormatterService;1"].
                            getService(Components.interfaces.koIFormatterService);
        var pref;
        var name;
        var menuitem;
        var formatter;
        var formatter_name;
        var menu_label;

        for (var j=0; j < formatter_configurations.length; j++) {
            pref = formatter_configurations[j];
            name = pref.getStringPref("name");
            formatter_name = pref.getStringPref("formatter_name");
            formatter = formatterSvc.getFormatterWithName(formatter_name);
            if (formatter) {
                menuitem = document.createElementNS(XUL_NS, 'menuitem');
                menuitem.setAttribute("id", "formatter_" + name);
                menuitem.setAttribute("dynamically_added_formatter", "true");
                if (region_name == "other") {
                    menu_label = name;
                } else if (region_name == "sublang") {
                    menu_label = strbundle.getFormattedString(region_name+".menu.prefix", [sublang, name]);
                } else {
                    menu_label = strbundle.getFormattedString(region_name+".menu.prefix", [name]);
                }
                menuitem.setAttribute('label', menu_label);
                menuitem.setAttribute('tooltiptext', formatter.prettyName + " (" + pref.getStringPref("lang") + ")");
                menuitem.setAttribute('oncommand', 'ko.formatters.formatViewUsingConfigUuid("' + pref.getStringPref("uuid") + '", ' +
                                      '"' + region_name + '", ' +
                                      pos_start + ', ' +
                                      pos_end + ');');
                if (addHighlighting) {
                    menuitem.setAttribute('onmouseover', 'ko.formatters.highlightRegion(' + pos_start + ', ' + pos_end + ');');
                    menuitem.setAttribute('onmouseout', 'ko.formatters.unhighlightRegion();');
                }
                menupopup.appendChild(menuitem);
            } else {
                this.log.warn("No formatter exists with the name: '" + formatter_name +
                              "', from preference '" + name + "'");
            }
        }
    };

    /**
     * Load the applicable formatter configurations into the menupopup, using
     * the currentView as the basis for determining the configuration language
     * and preferences.
     */
    function _createFormatterMenusForCurrentView(menupopup) {
        var view = ko.views.manager.currentView;
        if (!view) {
            return;
        }
        /* Remove any existing formatter menuitems. */
        var node = menupopup.firstChild;
        var nextSibling;
        while (node) {
            nextSibling = node.nextSibling;
            /* Only remove the menus that were previously added by this code. */
            /* Allows an overlay to add whatever they wish to this menu. */
            if (node.getAttribute("dynamically_added_formatter") == "true") {
                menupopup.removeChild(node);
            }
            node = nextSibling;
        }

        /**
         * @type {Components.interfaces.ISciMoz}
         */
        var scimoz = view.scimoz;

        var prefs = view.prefs;
        var lang = view.koDoc.language;
        var format_regions = [];

        // Work out the applicable formatter configurations.
        var all_formatter_configurations = ko.formatters.getAllFormatterConfigurationsForPrefset(prefs);
        var lang_formatter_configurations = ko.formatters.getFormatterConfigurationsForLangAndPrefset(lang, prefs);

        // Work out if there is a selection, that will be the first menu.
        var pos_start = scimoz.selectionStart;
        var pos_end = scimoz.selectionEnd;
        if (scimoz.selectionMode == scimoz.SC_SEL_LINES) {
            pos_start = scimoz.positionFromLine(scimoz.lineFromPosition(pos_start));
            pos_end = scimoz.getLineEndPosition(scimoz.lineFromPosition(pos_end));
        }
        var sublang = view.koDoc.languageForPosition(pos_start);

        if (pos_start != pos_end) {
            // There is a selection.
            // If the selection contains multiple languages, we only show the
            // language formatter, else we show the specific language for the
            // selection language.
            var selection_formatters;
            var points = {};
            var count = {};
            view.koDoc.getLanguageTransitionPoints(pos_start, pos_end, points, count);
            points = points.value;
            if (points.length > 2) {
                // There are multi-languages in the selection
                selection_formatters = lang_formatter_configurations;
            } else {
                selection_formatters = ko.formatters.getFormatterConfigurationsForLangAndPrefset(sublang, prefs);
            }
            format_regions.push(["selection", selection_formatters, points[0], points[0]]);
        }

        if (sublang && sublang != lang && sublang != "XML") {
            var points = {};
            var count = {};
            view.koDoc.getLanguageTransitionPoints(pos_start, pos_end, points, count);
            points = points.value;
            format_regions.push(["sublang", ko.formatters.getFormatterConfigurationsForLangAndPrefset(sublang, prefs), points[0], points[1]]);
        }

        format_regions.push(["document", lang_formatter_configurations, 0, scimoz.length]);

        var strbundle = document.getElementById("formatter_strings");
        //var other_localized_name = strbundle.getString("otherFormattersMenulistLabel");
        var other_localized_name = "";
        var other_localized_menu_label;
        var target_formatter_configurations;
        var other_formatter_configurations;
        var addBlockHighlighting;
        highlight_count = 0;
        for (i=0; i < format_regions.length; i++) {
            addBlockHighlighting = format_regions[i][0] == 'sublang';
            if (i > 0) {
                // Add a menu separator.
                if (all_formatter_configurations.length > 0) {
                    var menusep = document.createElementNS(XUL_NS, 'menuseparator');
                    menusep.setAttribute("id", "editor_contextmenu_formatter_other_sep");
                    menusep.setAttribute("dynamically_added_formatter", "true");
                    menupopup.appendChild(menusep);
                }
            }
            target_formatter_configurations = format_regions[i][1];
            //dump("target_formatter_configurations.length: " + target_formatter_configurations.length + "\n");
            if (target_formatter_configurations) {
                _createMenuitems(menupopup, strbundle,
                                 target_formatter_configurations,
                                 format_regions[i][0], sublang,
                                 format_regions[i][2], format_regions[i][3],
                                 addBlockHighlighting);
            }
    
            other_formatter_configurations = [];
            for (var j=0; j < all_formatter_configurations.length; j++) {
                if (target_formatter_configurations.indexOf(all_formatter_configurations[j]) < 0) {
                    other_formatter_configurations.push(all_formatter_configurations[j]);
                }
            }
            /* Add in other formatter menus/menuitems. */
            if (other_formatter_configurations.length > 0) {
                var other_menu = document.createElementNS(XUL_NS, 'menu');
                if (format_regions[i][0] == "sublang") {
                    other_localized_menu_label = strbundle.getFormattedString(format_regions[i][0]+".menu.prefix", [sublang, other_localized_name]);
                } else {
                    other_localized_menu_label = strbundle.getFormattedString(format_regions[i][0]+".menu.prefix", [other_localized_name]);
                }
                other_menu.setAttribute("label", other_localized_menu_label);
                other_menu.setAttribute("dynamically_added_formatter", "true");
                var other_menupopup = document.createElementNS(XUL_NS, 'menupopup');
                if (addBlockHighlighting) {
                    other_menupopup.setAttribute('onpopupshown', 'ko.formatters.highlightRegion(' + format_regions[i][2] + ', ' + format_regions[i][3] + ');');
                    other_menupopup.setAttribute('onpopuphiding', 'ko.formatters.unhighlightRegion();');
                }
                other_menu.appendChild(other_menupopup);
                _createMenuitems(other_menupopup, strbundle,
                                 other_formatter_configurations,
                                 "other", sublang,
                                 format_regions[i][2], format_regions[i][3],
                                 false /* highlighting */ );
                menupopup.appendChild(other_menu);
            }
        }
    };


    /************************************************************
     *        Exposed ko.formatters.XYZ functions               *
     ************************************************************/

    /**
     * Format this view. If there is a selection in this view or the
     * formatSelection argument is true, then the formatter will work upon the
     * view's selection, else the formatter works on the complete document.
     * The targetted text will be replaced if the formatting is successful.
     * @param view {Components.interfaces.koIScintillaView}
     *        (Optional) The view to format. Defaults to the currentview.
     * @param {int} format_type (Optional) the type of formatting context to
     *        use, one of (FORMAT_SELECTION, FORMAT_BLOCK, FORMAT_TEXT, ...)
     */
    this.formatView = function formatView(view, format_type /* everything */) {
        try {
            if (!view) {
                view = ko.views.manager.currentView;
            }
            if (!view || (view.getAttribute('type') != 'editor')) {
                return;
            }
            if ((typeof(format_type) == 'undefined') || (format_type == null)) {
                format_type = FORMAT_TEXT;
            }
            var formatterSvc = Components.classes["@activestate.com/koFormatterService;1"].
                                getService(Components.interfaces.koIFormatterService);
            var context = formatterSvc.createFormatterContextFromView(view);
            try {
                if (format_type == FORMAT_SELECTION && view.scimoz.selText) {
                    // Ensure the context is the selection.
                    context.type = FORMAT_SELECTION;
                    context.text = view.scimoz.selText;
                } else if (format_type == FORMAT_BLOCK) {
                    // Ensure it's the current sub-language block.
                    var sublang = view.koDoc.subLanguage;
                    if (sublang && sublang != view.koDoc.language && sublang != "XML") {
                        var points = {};
                        var count = {};
                        var scimoz = view.scimoz;
                        var currentPos = scimoz.currentPos;
                        view.koDoc.getLanguageTransitionPoints(currentPos, currentPos, points, count);
                        points = points.value;
                        context.type = FORMAT_BLOCK;
                        context.lang = sublang;
                        context.text = view.scimoz.getTextRange(points[0], points[1]);
                        context.pos_start = points[0];
                        context.pos_end = points[1];
                    }
                }
                var formatter = formatterSvc.getFormatterForContext(context);
                if (formatter) {
                    _formatViewWithFormatterAndContext(view, formatter, context);
                } else {
                    ko.statusBar.AddMessage("No formatter configured for language: " +
                                            view.koDoc.language,
                                            "formatting", 5000, true);
                }
            } finally {
                // Ensure the context is cleaned-up.
                context.text = "";
                context.scimoz = null;
                context.prefset = null;
                context.formatter_prefset = null;
            }
        } catch (ex) {
            log.exception(ex);
        }
    }

    /**
     * Format the sub-language at the current cursor position of the view.
     *
     * @param view {Components.interfaces.koIScintillaView}
     *        (Optional) The view to format. Defaults to the currentview.
     */
    this.formatSubLanguageBlock = function formatSubLanguageBlock(view) {
        this.formatView(view, FORMAT_BLOCK);
    }

    /**
     * Return a list of all formatter configuration preferences.
     * @returns {array}  List of koIPreferenceSet xpcom objects.
     */
    this.getAllFormatterConfigurationsForPrefset = function getAllFormatterConfigurationsForPrefset(prefset) {
        var configuration_prefsets = [];
        if (prefset.hasPref('configuredFormatters')) {
            var prefs = prefset.getPref('configuredFormatters');
            for (var i=0; i < prefs.length; i++) {
                configuration_prefsets.push(prefs.getPref(i));
            }
        }
        return configuration_prefsets;
    }

    /**
     * Return a list of formatter configuration preferences that are setup for
     * the given language.
     * @returns {array}  List of koIPreferenceSet xpcom objects.
     */
    this.getFormatterConfigurationsForLangAndPrefset = function getFormatterConfigurationsForLangAndPrefset(lang, prefset) {
        var all_configuration_prefsets = this.getAllFormatterConfigurationsForPrefset(prefset);
        var lang_configuration_prefsets = [];
        for (var i=0; i < all_configuration_prefsets.length; i++) {
            if (all_configuration_prefsets[i].getStringPref("lang") == lang) {
                lang_configuration_prefsets.push(all_configuration_prefsets[i]);
            }
        }
        return lang_configuration_prefsets;
    }

    /**
     * Return a list of formatter configuration preferences that are setup for
     * the current view's language and prefset.
     * @returns {array}  List of koIPreferenceSet xpcom objects.
     */
    this.getFormatterConfigurationsForView = function getFormatterConfigurationsForView(view) {
        if (!view) {
            view = ko.views.manager.currentView;
            if (!view) {
                return null;
            }
        }
        return this.getFormatterConfigurationsForLangAndPrefset(view.koDoc.language, view.prefs);
    }

    /**
     * Populate the provided menupopup element with the available formatters
     * for the current view.
     */
    this.loadFormatterMenusForCurrentView = function loadFormatterMenusForCurrentView(menupopup, event) {
        try {
            if (event.target && event.target.getAttribute("id") != menupopup.getAttribute("id")) {
                // Avoid loading sub-menu popups, as they are already loaded.
                return;
            }
            _createFormatterMenusForCurrentView(menupopup);
        } catch (ex) {
            log.exception(ex);
        }
    }

    /**
     * Format the given view (defaults to the current view when not supplied)
     * with the formatter configuration provided. If there is a selection in
     * this view then the formatter will work upon the view's selection, else
     * the formatter works on the complete document. The targetted text will be
     * replaced if the formatting is successful.
     * 
     * @param view {Components.interfaces.koIScintillaView}
     *        The view to format. Defaults to the current view if null.
     * @param config {Components.interfaces.koIPreferenceSet}
     *        The formatter configuration prefset to use for formatting.
     * @param {string} format_type  Must be one of ["document", "sublang", "selection"]
     * @param {int} pos_start  (Optional) The position to start the formatting.
     * @param {int} pos_end  (Optional) The position to end the formatting.
     */
    this.formatViewUsingConfig = function formatViewUsingConfig(view, config, format_type, pos_start, pos_end) {
        log.debug("formatting view using: " + config.getStringPref("name"));
        var formatterSvc = Components.classes["@activestate.com/koFormatterService;1"].
                            getService(Components.interfaces.koIFormatterService);
        var formatter = formatterSvc.getFormatterWithName(config.getStringPref("formatter_name"));
        if (formatter) {
            var context;
            var context = formatterSvc.createFormatterContextFromView(view);
            try {
                if (format_type == "selection" &&
                    (context.type != FORMAT_SELECTION)) {
                    var selText = view.scimoz.selText;
                    if (selText) {
                        context.type = FORMAT_SELECTION;
                        context.text = selText;
                    }
                } else if (format_type == "sublang") {
                    context.type = FORMAT_BLOCK;
                    context.text = view.scimoz.getTextRange(pos_start, pos_end);
                    context.pos_start = pos_start;
                    context.pos_end = pos_end;
                }

                context.formatter_prefset = config;
                _formatViewWithFormatterAndContext(view, formatter, context);
            } finally {
                // Ensure the context instance is cleaned-up.
                context.text = "";
                context.prefset = null;
                context.formatter_prefset = null;
                context.scimoz = null;
            }
        } else {
            ko.dialogs.alert("No formatter exists with the name: '" +
                             formatter_name + "', from preference '" +
                             name + "'",
                             null,
                             "Invalid Formatter Configuration");
        }
    }

    /**
     * Format the given view (defaults to the currentView when not supplied)
     * with the formatter configuration that has the supplied config name.
     * If there is a selection in this view then the formatter will work upon
     * the view's selection, else the formatter works on the complete document.
     * The targetted text will be replaced if the formatting is successful.
     
     * @param view {Components.interfaces.koIScintillaView}
     *        The view to format. Defaults to the current view if null.
     * @param {string} configName
     *        The name of the formatter configuration to use for formatting.
     */
    this.formatViewUsingConfigName = function formatViewUsingConfigName(view, configName) {
        try {
            if (!view) {
                view = ko.views.manager.currentView;
                if (!view) {
                    return;
                }
            }
            // Work out the applicable formatter configurations.
            var all_formatter_configurations = this.getAllFormatterConfigurationsForPrefset(view.prefs);
            for (var i=0; i < all_formatter_configurations.length; i++) {
                if (all_formatter_configurations[i].getStringPref("name") == configName) {
                    this.formatViewUsingConfig(view, all_formatter_configurations[i]);
                    break;
                }
            }
        } catch (ex) {
            log.exception(ex);
        }
    },

    /**
     * Format the current view with the formatter configuration that has the
     * provided UUID. This is required for the formatter menu(item)'s.
     * 
     * @param {string} uuid  The unique identifier of the configuration prefset.
     * @param {string} format_type  Must be one of ["document", "sublang", "selection"]
     * @param {int} pos_start  (Optional) The position to start the formatting.
     * @param {int} pos_end  (Optional) The position to end the formatting.
     */
    this.formatViewUsingConfigUuid = function formatViewUsingConfigUuid(uuid, format_type, pos_start, pos_end) {
        try {
            var view = ko.views.manager.currentView;
            if (!view) {
                return;
            }
            // Work out the applicable formatter configurations.
            var all_formatter_configurations = this.getAllFormatterConfigurationsForPrefset(view.prefs);
            for (var i=0; i < all_formatter_configurations.length; i++) {
                if (all_formatter_configurations[i].getStringPref("uuid") == uuid) {
                    this.formatViewUsingConfig(view,
                                               all_formatter_configurations[i],
                                               format_type, pos_start, pos_end);
                    break;
                }
            }
        } catch (ex) {
            log.exception(ex);
        }
    }

    // Controller instance on the current window to handle formatter commands.
    window.controllers.appendController(new FormattersController());

}).apply(ko.formatters);
